AJS.$(document).ready(function() {

    VM$.configuration.getShowUserPickerVacationWarning(function(doShow) {
        if (doShow) {
            var userVacationInfo = new UserVacationInfo();

            findUserPickerFields(userVacationInfo);
            findUserFields(userVacationInfo);
            JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
                if (reason == 'dialogContentReady' || reason == 'dialogReady') {
                    findUserPickerFields(userVacationInfo);
                    findUserFields(userVacationInfo);
                }
            });

            JIRA.bind(JIRA.Events.PANEL_REFRESHED, function(e, context, reason) {
                findUserPickerFields(userVacationInfo);
                findUserFields(userVacationInfo);
            });

            JIRA.bind(JIRA.Events.ISSUE_REFRESHED, function(e, context, reason) {
                findUserPickerFields(userVacationInfo);
                findUserFields(userVacationInfo);
            });
        }
    });
});

function UserVacationInfo() {
    var ajaxCacher = new AjaxCacher(AJS.$);

    function getCurrentVacationInfoForUser(userName, responseCallback) {
        var params = new Object();
        params.userName = userName;

        ajaxCacher.ajax({
            type : "POST",
            url : AJS.contextPath() + "/rest/vacation-manager/1.0/user-vacation/is-on-vacation",
            data : JSON.stringify(params),
            dataType : "json",
            contentType : 'application/json; charset=utf-8',
            async : true,
            cacheId : userName
        }, responseCallback);
    }

    return {
        getCurrentVacationInfoForUser : getCurrentVacationInfoForUser
    }
}

function findUserPickerFields(userVacationInfo) {
    AJS.$("input.userpickerfield,select.single-user-picker").each(function() {
        var obj = AJS.$(this);
        if (obj.attr("vm-inited") != "true") {
            obj.attr("vm-inited", "true");
            try {
                new UserPickerWarning(obj, userVacationInfo);
            } catch (ex) {
                console.log("Cannot parse user picker field: " + ex);
            }
        }
    });
}

function UserPickerWarning(field, userVacationInfo) {
    var messageContainer = null;

    function initMessageContainer() {
        messageContainer = $("<div class='vm-warning-field-group'></div>");
        field.closest("div.field-group").append(messageContainer);
        messageContainer.hide();
    }

    field.on('change', function() {
        checkUser(getUserName());
    });

    function checkUser(userName) {
    	if (!userName) {
    		hideWarningMessage();
    		return;
    	}

        userVacationInfo.getCurrentVacationInfoForUser(userName, function(data) {
            if (data && data.hasVacation) {
                showWarningMessage(new Date(data.hasVacationSince), new Date(data.hasVacationUntil), data.substitute);
            } else {
                hideWarningMessage();
            }
        });
    }
    
    function hideWarningMessage() {
        messageContainer.hide();
    }

    function showWarningMessage(dateStart, dateEnd, substitute) {
        messageContainer.html(getUserPickerWarningMessage(dateStart, dateEnd));
        if (substitute) {
            messageContainer.html(messageContainer.html()+AJS.I18n.getText("vacationmanager.userpicker.warning.substituted.by.one.line", substitute.displayName, substitute.name));
        }
        messageContainer.show();
    }

    function getUserName() {
        return field.val().toString();
    }

    initMessageContainer();
    checkUser(getUserName());
}

function findUserFields(userVacationInfo) {
    AJS.$("[data-user][rel],[data-fieldtype='userpicker']").each(function() {
        var obj = AJS.$(this);
        if (obj.attr("vm-inited") != "true") {
            obj.attr("vm-inited", "true");
            try {
                new UserFieldWarning(obj, userVacationInfo);
            } catch (ex) {
                console.log("Cannot parse user field: " + ex);
            }
        }
    });
}

function UserFieldWarning(field, userVacationInfo) {
    var userName = field.attr("rel");
    if (userName == undefined) {
        userName = field.find("[rel]").attr("rel");
    }
    
    if (!userName) {
    	return;
    }

    userVacationInfo.getCurrentVacationInfoForUser(userName, function(data) {
        if (data && data.hasVacation) {
            addWarningMessage(new Date(data.hasVacationSince), new Date(data.hasVacationUntil), data.substitute);
        }
    });

    function addWarningMessage(dateStart, dateEnd, substitute) {
        var messageContainer = $("<div class='vm-warning-field-group'></div>");
        messageContainer.html("<div>"+getUserPickerWarningMessage(dateStart, dateEnd)+"</div>");
        if (substitute) {
            messageContainer.html(messageContainer.html()+"<div>"+AJS.I18n.getText("vacationmanager.userpicker.warning.substituted.by", substitute.displayName, substitute.name)+"</div>");
        }
        field.after(messageContainer);
    }
}

function getUserPickerWarningMessage(dateStart, dateEnd) {
    var daysUntilLeave = moment(dateStart).startOf('day').diff(moment().startOf('day'), 'days');
    if (daysUntilLeave == 1) {
        return AJS.I18n.getText("vacationmanager.userpicker.warning.upcoming.leave.tomorow", VM$.formatDate(dateStart), VM$.formatDate(dateEnd));
    } else if (daysUntilLeave > 0) {
        return AJS.I18n.getText("vacationmanager.userpicker.warning.upcoming.leave", daysUntilLeave, VM$.formatDate(dateStart), VM$.formatDate(dateEnd));
    }
    return AJS.I18n.getText("vacationmanager.userpicker.warning.has.vacation.until", VM$.formatDate(dateEnd));
}
